@@ -17,10 +17,10 @@ from . import constants
 from .compat import FileNotFoundError
 from ._ffi import ffi
 
-VERSION = '0.8.0'
-# pycairo compat:
-version = '1.10.0'
-version_info = (1, 10, 0)
+VERSION = '0.9.0'
+# supported version of cairo, used to be pycairo version too:
+version = '1.16.0'
+version_info = (1, 16, 0)
 
 
 def dlopen(ffi, *names):
@@ -38,7 +38,7 @@ def dlopen(ffi, *names):
     raise OSError("dlopen() failed to load a library: %s" % ' / '.join(names))
 
 
-cairo = dlopen(ffi, 'cairo', 'cairo-2')
+cairo = dlopen(ffi, 'cairo', 'cairo-2', 'cairo-gobject-2')
 
 
 class _keepref(object):
