@@ -1,4 +1,4 @@
-# coding: utf8
+# coding: utf-8
 """
     cairocffi.tests
     ~~~~~~~~~~~~~~~
@@ -97,9 +97,9 @@ def test_image_bytearray_buffer():
     assert data == pixel(b'\x80\x00\x00\x00') * 200
 
 
+@pytest.mark.xfail(cairo_version() < 11200,
+                   reason='Cairo version too low')
 def test_surface_create_similar_image():
-    if cairo_version() < 11200:
-        pytest.xfail()
     surface = ImageSurface(cairocffi.FORMAT_ARGB32, 20, 30)
     similar = surface.create_similar_image(cairocffi.FORMAT_A8, 4, 100)
     assert isinstance(similar, ImageSurface)
@@ -109,9 +109,9 @@ def test_surface_create_similar_image():
     assert similar.get_height() == 100
 
 
+@pytest.mark.xfail(cairo_version() < 11000,
+                   reason='Cairo version too low')
 def test_surface_create_for_rectangle():
-    if cairo_version() < 11000:
-        pytest.xfail()
     surface = ImageSurface(cairocffi.FORMAT_A8, 4, 4)
     data = surface.get_data()
     assert data[:] == b'\x00' * 16
@@ -183,9 +183,9 @@ def test_target_lifetime():
     assert sys.getrefcount(target) == initial_refcount
 
 
+@pytest.mark.xfail(cairo_version() < 11000,
+                   reason='Cairo version too low')
 def test_mime_data():
-    if cairo_version() < 11000:
-        pytest.xfail()
     surface = PDFSurface(None, 1, 1)
     assert surface.get_mime_data('image/jpeg') is None
     gc.collect()  # Clean up KeepAlive stuff from other tests
@@ -202,16 +202,138 @@ def test_mime_data():
     surface.finish()
     assert_raise_finished(surface.set_mime_data, 'image/jpeg', None)
 
-
+@pytest.mark.xfail(cairo_version() < 11200,
+                   reason='Cairo version too low')
 def test_supports_mime_type():
-    if cairo_version() < 11200:
-        pytest.xfail()
     # Also test we get actual booleans:
     assert PDFSurface(None, 1, 1).supports_mime_type('image/jpeg') is True
     surface = ImageSurface(cairocffi.FORMAT_A8, 1, 1)
     assert surface.supports_mime_type('image/jpeg') is False
 
 
+@pytest.mark.xfail(cairo_version() < 11400,
+                   reason='Cairo version too low')
+def test_device_scale():
+    surface = PDFSurface(None, 1, 1)
+    assert surface.get_device_scale() == (1, 1)
+    surface.set_device_scale(2, 3)
+    assert surface.get_device_scale() == (2, 3)
+
+
+@pytest.mark.xfail(cairo_version() < 11504,
+                   reason='Cairo version too low')
+def test_metadata():
+    file_obj = io.BytesIO()
+    surface = PDFSurface(file_obj, 1, 1)
+    surface.set_metadata(PDF_METADATA_TITLE, 'title')
+    surface.set_metadata(PDF_METADATA_SUBJECT, 'subject')
+    surface.set_metadata(PDF_METADATA_CREATOR, 'creator')
+    surface.set_metadata(PDF_METADATA_AUTHOR, 'author')
+    surface.set_metadata(PDF_METADATA_KEYWORDS, 'keywords')
+    surface.set_metadata(PDF_METADATA_CREATE_DATE, '2013-07-21T23:46:00+01:00')
+    surface.set_metadata(PDF_METADATA_MOD_DATE, '2013-07-21T23:46:00Z')
+    surface.finish()
+    pdf_bytes = file_obj.getvalue()
+    assert b'/Title (title)' in pdf_bytes
+    assert b'/Subject (subject)' in pdf_bytes
+    assert b'/Creator (creator)' in pdf_bytes
+    assert b'/Author (author)' in pdf_bytes
+    assert b'/Keywords (keywords)' in pdf_bytes
+    assert b"/CreationDate (20130721234600+01'00)" in pdf_bytes
+    assert b'/ModDate (20130721234600Z)' in pdf_bytes
+
+
+@pytest.mark.xfail(cairo_version() < 11504,
+                   reason='Cairo version too low')
+def test_outline():
+    file_obj = io.BytesIO()
+    surface = PDFSurface(file_obj, 1, 1)
+    outline = surface.add_outline(
+        PDF_OUTLINE_ROOT, 'title 1', 'page=1 pos=[1 1]',
+        PDF_OUTLINE_FLAG_OPEN & PDF_OUTLINE_FLAG_BOLD)
+    surface.add_outline(outline, 'title 2', 'page=1 pos=[1 1]')
+    surface.finish()
+    pdf_bytes = file_obj.getvalue()
+    assert b'/Title (title 1)' in pdf_bytes
+    assert b'/Title (title 2)' in pdf_bytes
+
+
+@pytest.mark.xfail(cairo_version() < 11504,
+                   reason='Cairo version too low')
+def test_page_label():
+    file_obj = io.BytesIO()
+    surface = PDFSurface(file_obj, 1, 1)
+    surface.set_page_label('abc')
+    surface.finish()
+    pdf_bytes = file_obj.getvalue()
+    assert b'/P (abc)' in pdf_bytes
+
+
+@pytest.mark.xfail(cairo_version() < 11504,
+                   reason='Cairo version too low')
+def test_tag():
+    file_obj = io.BytesIO()
+    surface = PDFSurface(file_obj, 10, 10)
+    context = Context(surface)
+    context.tag_begin('Document')
+    context.tag_begin(
+        TAG_LINK,
+        attributes='rect=[1 2 4 5] uri=\'https://cairocffi.readthedocs.io/\'')
+    context.set_source_rgba(1, 0, .5, 1)
+    context.rectangle(2, 3, 4, 5)
+    context.fill()
+    context.tag_end(TAG_LINK)
+    context.tag_end('Document')
+    context.show_page()
+    surface.finish()
+    pdf_bytes = file_obj.getvalue()
+    assert b'/URI (https://cairocffi.readthedocs.io/)' in pdf_bytes
+    assert b'/S /Document' in pdf_bytes
+
+
+@pytest.mark.xfail(cairo_version() < 11504,
+                   reason='Cairo version too low')
+def test_thumbnail_size():
+    file_obj = io.BytesIO()
+    surface = PDFSurface(file_obj, 1, 1)
+    surface.set_thumbnail_size(1, 1)
+    surface.finish()
+    pdf_bytes1 = file_obj.getvalue()
+
+    file_obj = io.BytesIO()
+    surface = PDFSurface(file_obj, 1, 1)
+    surface.set_thumbnail_size(9, 9)
+    surface.finish()
+    pdf_bytes2 = file_obj.getvalue()
+
+    assert len(pdf_bytes1) < len(pdf_bytes2)
+
+
+@pytest.mark.xfail(cairo_version() < 11510,
+                   reason='Cairo version too low')
+def test_document_unit():
+    surface = SVGSurface(None, 1, 2)
+    assert surface.get_document_unit() == SVG_UNIT_PT
+
+    file_obj = io.BytesIO()
+    surface = SVGSurface(file_obj, 1, 2)
+    surface.set_document_unit(SVG_UNIT_PX)
+    assert surface.get_document_unit() == SVG_UNIT_PX
+    surface.finish()
+    pdf_bytes = file_obj.getvalue()
+    assert b'width="1px"' in pdf_bytes
+    assert b'height="2px"' in pdf_bytes
+
+    file_obj = io.BytesIO()
+    surface = SVGSurface(file_obj, 1, 2)
+    surface.set_document_unit(SVG_UNIT_PC)
+    assert surface.get_document_unit() == SVG_UNIT_PC
+    surface.finish()
+    pdf_bytes = file_obj.getvalue()
+    assert b'width="1pc"' in pdf_bytes
+    assert b'height="2pc"' in pdf_bytes
+
+
 def test_png():
     png_bytes = base64.b64decode(
         b'iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAADUlEQVQI12O'
@@ -255,9 +377,9 @@ def test_png():
         surface = ImageSurface.create_from_png(io.BytesIO(b''))
 
 
+@pytest.mark.xfail(cairo_version() < 11000,
+                   reason='Cairo version too low')
 def test_pdf_versions():
-    if cairo_version() < 11000:
-        pytest.xfail()
     assert set(PDFSurface.get_versions()) >= set([
         cairocffi.PDF_VERSION_1_4, cairocffi.PDF_VERSION_1_5])
     assert PDFSurface.version_to_string(cairocffi.PDF_VERSION_1_4) == 'PDF 1.4'
@@ -292,7 +414,8 @@ def test_pdf_surface():
         pdf_bytes = file_obj.getvalue()
         assert pdf_bytes.startswith(b'%PDF')
         assert b'/MediaBox [ 0 0 123 432 ]' in pdf_bytes
-        assert pdf_bytes.count(b'/Type /Page\n') == 1
+        assert pdf_bytes.count(b'/Type /Pages') == 1
+        assert pdf_bytes.count(b'/Type /Page') == 2
 
     file_obj = io.BytesIO()
     surface = PDFSurface(file_obj, 1, 1)
@@ -306,7 +429,8 @@ def test_pdf_surface():
     assert b'/MediaBox [ 0 0 1 1 ]' not in pdf_bytes
     assert b'/MediaBox [ 0 0 12 100 ]' in pdf_bytes
     assert b'/MediaBox [ 0 0 42 700 ]' in pdf_bytes
-    assert pdf_bytes.count(b'/Type /Page\n') == 2
+    assert pdf_bytes.count(b'/Type /Pages') == 1
+    assert pdf_bytes.count(b'/Type /Page') == 3
 
 
 def test_svg_surface():
@@ -379,9 +503,9 @@ def test_ps_surface():
     assert b'%%dolor' in ps_bytes
 
 
+@pytest.mark.xfail(cairo_version() < 11000,
+                   reason='Cairo version too low')
 def _recording_surface_common(extents):
-    if cairo_version() < 11000:
-        pytest.xfail()
     surface = ImageSurface(cairocffi.FORMAT_ARGB32, 100, 100)
     empty_pixels = surface.get_data()[:]
     assert empty_pixels == b'\x00' * 40000
@@ -415,17 +539,16 @@ def test_recording_surface():
     assert recorded_pixels == text_pixels
 
 
+@pytest.mark.xfail(cairo_version() < 11200,
+                   reason='Cairo version too low')
 def test_unbounded_recording_surface():
-    if cairo_version() < 11200:
-        # Unbounded recording surfaces do not seem to record anything on 1.10.
-        pytest.xfail()
     text_pixels, recorded_pixels = _recording_surface_common(None)
     assert recorded_pixels == text_pixels
 
 
+@pytest.mark.xfail(cairo_version() < 11200,
+                   reason='Cairo version too low')
 def test_recording_surface_get_extents():
-    if cairo_version() < 11200:
-        pytest.xfail()
     for extents in [None, (0, 0, 140, 80)]:
         surface = RecordingSurface(cairocffi.CONTENT_COLOR_ALPHA, extents)
         assert surface.get_extents() == extents
@@ -865,9 +988,9 @@ def test_context_clip():
     assert context.clip_extents() == (1, 1, 3, 4)
 
 
+@pytest.mark.xfail(cairo_version() < 11000,
+                   reason='Cairo version too low')
 def test_context_in_clip():
-    if cairo_version() < 11000:
-        pytest.xfail()
     surface = ImageSurface(cairocffi.FORMAT_A8, 4, 4)
     context = Context(surface)
     context.rectangle(1, 1, 2, 2)
@@ -945,7 +1068,7 @@ def test_context_font():
     ascent, descent, height, max_x_advance, max_y_advance = (
         context.font_extents())
     # That’s about all we can assume for a default font.
-    #assert height > ascent + descent  # Not even this is true on all fonts
+#    assert height > ascent + descent  # Not even this is true on all fonts
     assert max_x_advance > 0
     assert max_y_advance == 0
     _, _, _, _, x_advance, y_advance = context.text_extents('i' * 10)
@@ -1050,6 +1173,19 @@ def test_font_options():
     assert options_2 == options_1
 
 
+@pytest.mark.xfail(cairo_version() < 11512,
+                   reason='Cairo version too low')
+def test_font_options_variations():
+    options = FontOptions()
+
+    assert options.get_variations() is None
+    options.set_variations('wght 400, wdth 300')
+    assert options.get_variations() == 'wght 400, wdth 300'
+    options.set_variations(None)
+    assert options.get_variations() is None
+
+
+
 def test_glyphs():
     surface = ImageSurface(cairocffi.FORMAT_ARGB32, 100, 20)
     context = Context(surface)
@@ -1064,8 +1200,8 @@ def test_glyphs():
     assert 5 == x1 < x2 < x3
     assert clusters == [(2, 1), (1, 1), (1, 1)]
     assert is_backwards == 0
-    assert font.glyph_extents(glyphs) == font.text_extents(text)
-    assert font.glyph_extents(glyphs) == context.glyph_extents(glyphs)
+    assert round_tuple(font.glyph_extents(glyphs)) == round_tuple(font.text_extents(text))
+    assert round_tuple(font.glyph_extents(glyphs)) == round_tuple(context.glyph_extents(glyphs))
 
     assert context.copy_path() == []
     context.glyph_path(glyphs)
